/* heater.c */
/*       10        20        30        40        50        60        70       */
/*3456789012345678901234567890123456789012345678901234567890123456789012345671*/
/*----------------------------------------------------------------------------*/

#include "global.h" 
#include <stdio.h>
#include <stdlib.h>
#include <math.h>

/*       10        20        30        40        50        60        70       */
/*3456789012345678901234567890123456789012345678901234567890123456789012345671*/
/*
000213 internal energy balance included on pedantic flag (use only 1 heater)
010523 out labels added
020404 syntax edit, mode 2 , exit on Tset=0 added
*/

/*-------------------------------------------------------AUX HEATER TYPE 6----*/
int heater(unit,par,in_u,in_nr,out_o,out_n,deriv_o,deriv_n,h,init_energy,
           qp_sum,tsim,init_flag)
           int    unit;      /* current unit number */
           double **par;     /* parameter array par[unit][1..PARMAX]  */
           int    **in_u;    /* input unit array, input connections   */
           int    **in_nr;   /* input number array                    */
           double **out_o;   /* output array with privious state      */
           double **out_n;   /* output array with current state       */
           double **deriv_o; /* derivatives: used for diffential equ. */
           double **deriv_n; /*  new state of deriv                   */
           double h;         /* sim time step                         */
           double **init_energy; /* initial energy in RC-comp. [Ws]   */
           double ***qp_sum; /* energy to one node qp[unit][n][m]     */
           double tsim;      /* simulation time in [s] of this day    */
           int    init_flag; /* set at initial loop of sim            */

{
/* etc/xfig.awk        
   
   TYPE 06
   PRIORITY  6          used by config routine
   HYDRAULIC 1          1-pole fluid            

   
   PAR1: 30000 [W] 'Pmax' maximum heating rate
   PAR2: 70 [C] 'Tset' mode 0:set temp,mode 1: dT_hys_off
   PAR3: 4.19 [kJ/kgK] 'cp' Specific heat of fluid cp [kJ/kgK]
   PAR4: 3 [1] 'mode' 0:Tset par2,1:mod,2:Tset by inp3,3:constPow 4:pow inp3
   PAR5: 1 [1] 'cond' 2: use CondensGainHeater table for Paux
   PAR6: 0.3 [1] 'ctr_min' Pmin/Pmax minimum HeatPower
   
   INP1: 0 [Ws] 'dQin' deltaQ-inlet
   INP2: 0 [kg/s] 'mdotin' mp-inlet
   INP3: 1 [1] 'ctr' control/setvalue see par4: mode 
   INP4: 20.0 [C] 'Tamb' ambient temperature 

   OUT1: [Ws] 'dQout' deltaQ-outlet
   OUT2: [kg/s] 'mp' mp-outlet
   OUT3: [C] 'Tout' temperature outlet
   OUT4: [W] 'P' effectiv power for heating, eta=1
   OUT5: [C] 'Tin' inlet
   OUT6: [Ws] 'dQgain' system inlet energy 
   OUT7: [Ws] 'dQloss' system outlet energy 
   OUT8: [Ws] 'Qcap' internal energy 
   OUT9: [Ws] 'Qaux' due to primary energy            
   OU10: [Ws] 'P'    due to primary energy
   OU11: [1] 'n-starts' number of starts

*/
  int a=0; /* debug with a=1 */
  int pedantic=0;

  double Pmax=      par[unit][1];
  double cp  =      par[unit][3]*1e+3; /* Ws/K */
  int    mode=(int) par[unit][4]; /* mode of heater */

  double Qin    =out_n IN(1);
  double mp_cp_h=out_n IN(2)*cp*h;
  double Tamb   =out_n IN(4);         

  double Qout,Qaux,Tset,Tout=0;
  double eta=1;
  double T_RL=Qin/(mp_cp_h+1e-12); /* temp in */

/*---------init---------------------------------------------------------------*/
if (init_flag) {
  if (mode>4) { printf("wrong heater-mode u:%d\n",unit); exit (-1); }
}

/*----------------------------------------------------------------------------*/
  /*------------------------mode0 : heater-----------------*/
if ( mode==0 )  { /* mode 0: Tset by par2 with maximum power par1 if ctr=1 */
  double Tset=par[unit][2];
  double ctr=out_n IN(3); 
  if ( ( ctr == 1 ) && (Qin != 0) ) Qout=mp_cp_h*Tset; /* set temp  */
  else                              Qout=Qin;  /* no heating */

  Qaux=Qout-Qin; /*Q_aux=dQ_out-dQ_in */
  if ( Qaux > (Pmax*h) ) { /* maximum power */
      Qaux=Pmax*h;
      Qout=Qaux+Qin;
  } /* maximum power */ 
  if (mp_cp_h==0) Tout=Tset;
} /* mode0 */
  
/*----------------------------------------------------------------------------*/
  /*------------------------mode1: MODULA1-----------------*/
if ( mode==1) { /* mode 1*/
       /*
       modula-haet: heat to Tset,not more than Pmax;but more than Pmin
                    do no cooling;switch off heat at Tset+deltaT_off
       par1: max heat
       par2: deltaT_off: heater off, if TKV>Set_temp+deltaT_off
       par6: Pmin/Pmax 0..1
       inp3: Set-temperature
       */   
 double Pmax      =par[unit][1];
 double deltaT_off=par[unit][2];
 double ctr_min   =par[unit][6];

 Tset=out_n IN(3); 

 if (ctr_min==0) { printf("set paramter6 of heater:%d \n",unit); exit(-1);}

 if (Qin != 0) Qout=mp_cp_h*Tset;
 else          Qout=Qin;
  
 Qaux=Qout-Qin; 
 if (Qaux>(Pmax*h) )  { /* maximum power */
  Qaux=Pmax*h;
  Qout=Qaux+Qin;
 }
 if (Qaux<0) { /* no thermostat */
  Qaux=0;
  Qout=Qaux+Qin;
 }
 if (Qaux<(ctr_min*Pmax*h) && (Qaux>0) ) { /* minimum power */
  Qaux=ctr_min*Pmax*h;
  Qout=Qaux+Qin;
 }
 if (mp_cp_h>0) Tout=Qout/(mp_cp_h); 
 else           Tout=Tset;
 if (Tout>Tset+deltaT_off) { /* if heater hot , no power !! */
  Qaux=0;
  Qout=Qaux+Qin;
 }
 if (a) printf("heater[%d]: tsim:%.1f Tset%.1f Tout:%.1f Tin:%.1f  "
                   "Q_aux:%.1f \n"
                   ,unit,tsim,Tset,Tout ,Qin/(mp_cp_h+1e-12),Qaux);
} /* mode 1 */
    
  /*------------------------mode2 : thermostat set by Tset=contr inp3 */
if ( mode==2) {
  double Tset=out_n IN(3);
  if (Qin != 0) {
    Qout=mp_cp_h*Tset; /*Tset =inp3 */      
    if (Qout<=0) { 
      printf("heater:%d,Tset:%.1f mp:%.1f kg/h\n",unit,Tset,out_n IN(2)*3600);
    }
  } /* (Qin != 0) */
  else Qout=Qin;

  if ( (Tset<=1e-2) && (!init_flag) ) { 
    printf("heater[%d] Tset = 0 !!! :%.1f\n",unit,Tset);
    exit(-1);
  }
  Qaux=Qout-Qin; /* could be negative */
  if (mp_cp_h==0) Tout=Tset;
  if (a) printf("heater[%d]: Tset:%.1f Paux:%.1f mdot:%.1fkg/h Qout:%.1fWs \n"
     ,unit,Tset,Qaux/h,out_n IN(2),Qout);
} /* mode2 */

/*----------------------------------------------------------------------------*/
  /*------------------------mode3 : constant power---------*/
if ( mode==3 ) { /* mode 3: heater */
 double ctr=out_n IN(3);

 if ( ( ctr == 1 ) && (Qin != 0) ) Qout=Qin+Pmax*h;  
 else                              Qout=Qin;  /* no heating */

 Qaux=Qout-Qin; /*Q_aux=dQ_out-dQ_in */
 if ( Qaux > (Pmax*h) ) { /* maximum power */
   Qaux=Pmax*h;
   Qout=Qaux+Qin;
 }
if (mp_cp_h==0) Tout=Tamb;
} /* mode 3 */

/*------input power mode 4----------------------------------------------------*/
if ( mode==4 ) { /* mode 4: power from inp3 */
 double power=out_n IN (3);
 if (mp_cp_h>0) Qout=Qin+power*h;
 else           Qout=Qin;

 Qaux=Qout-Qin; /*Q_aux=dQ_out-dQ_in */
 if ( Qaux > (Pmax*h) ) { /* maximum power */
   printf("heater:%d mode4 p_set:%.1f > Pmax:%.1f\n",unit,power,Pmax);
   Qaux=Pmax*h;
   Qout=Qaux+Qin;
 }
 if (mp_cp_h==0) Tout=Tamb;
} /* end mode 4 */
  

if (   ( (Qout==0) && (Qin!=0) )  /* check plug flow */
      || (Qout<0) 
   ) printf("heater:%d mode:%d Qin:%.1f Qout:%.1f\n",unit,mode,Qin,Qout);

/*----------------------------------------------------------------------------*/
/*--------------set outputs----------------------*/

out_n[unit][1]=Qout;   
out_n[unit][2]=out_n IN(2);

if ( (mp_cp_h>0) && (Qout!=0) ) out_n[unit][3]=Qout/(mp_cp_h); /* T_out */
else                            out_n[unit][3]=Tout;

out_n[unit][4]=(out_n[unit][1]-Qin)/h; /*P_aux=Qdiff/h */

if ( (mp_cp_h>0) && (Qout>0) ) out_n[unit][5]=Qin/(mp_cp_h); /*T_in */
else out_n[unit][5]=Tamb;

  /*System Energy */
out_n[unit][6]=Qaux;
 /* out7=0; no system losses     */
 /* out8=0: no internal capacity */

out_n[unit][9]=fabs(Qaux); /* Q heater */

   /*  check prints for eta function for (i=0;i<=100;i++) */
if (par[unit][5]==2) /* select CondensGainHeater */
   {
   switch (10*(int) (T_RL*0.1) ) /* set eta field */
    {
    case 70: eta=96   -0.1 *(T_RL-70); break;
    case 60: eta=97   -0.1 *(T_RL-60); break;
    case 50: eta=100  -0.3 *(T_RL-50); break;
    case 40: eta=103.5-0.35*(T_RL-40); break;
    case 30: eta=105.5-0.2 *(T_RL-30); break;
    case 20: eta=106  -0.05*(T_RL-20); break;
    case 10: eta=106.5-0.05*(T_RL-10); break;
    default: eta=95;                   break;
    }
   out_n[unit][9]=fabs(Qaux*100/eta);  /* Q heater */
} /* CondensGainHeater */

out_n[unit][10]=out_n[unit][9]/h; /* P , due to primary power */

   /* number of heater starts */
if ( (out_n[unit][2]>0) && (out_o[unit][2]==0) ) 
   out_n[unit][11]=out_o[unit][11]=out_o[unit][11]+1;
   

if (pedantic) { /* unit energy check */
    static double Qunit_transfer_sum=0,Qinit=0,Qsys_losses_sum=0;
    double Qunit_transfer,Qsys_losses,energy_change;

    energy_change=out_n[unit][8]-out_o[unit][8];
    Qunit_transfer= out_n IN (1)-out_n[unit][1];
    Qsys_losses=-out_n[unit][6];
    if ( ( fabs( Qunit_transfer-Qsys_losses-energy_change)>1 ) && (!init_flag) ) {
      printf("heater: energy_check error tsim:%.1f Qunit_transfer:%.1f "
             "Qsys_losses:%.1f energy_change:%.1f error:%.1f \n",
             tsim,Qunit_transfer,Qsys_losses,energy_change,
             Qunit_transfer-Qsys_losses-energy_change);
      printf("in1:%.1f out1:%.1f \n", out_n IN (1),out_n[unit][1]);
      exit (-1);
    } /* error */
    /* history energy check */
    if (init_flag) Qinit=out_n[unit][8]+Qsys_losses;
    Qunit_transfer_sum+=Qunit_transfer;
    Qsys_losses_sum+=Qsys_losses;
    if ( ( fabs( Qinit+Qunit_transfer_sum-Qsys_losses_sum-out_n[unit][8])>1 ) 
          && (!init_flag) ) {
       printf("heater: history energy_check error tsim:%.1f Qinit:%.1f "
             "Qunit_transfer_sum:%.1f "
             "Qsys_losses_sum:%.1f sys_energy:%.1f error:%.1f\n",
             tsim,Qinit,Qunit_transfer_sum,Qsys_losses_sum,out_n[unit][8],
             Qinit+Qunit_transfer_sum-Qsys_losses_sum-out_n[unit][8]);
      exit (-1);
    } /* error */
} /* unit energy check */

  return 1;
} /* heater */
/*----------------------------------------------------------------------------*/

